/*
    PocketCellar

    Copyright (C) 2002 George Staikos <staikos@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <qmessagebox.h>


#include <stdio.h>
#include <stdlib.h>

#include <qlineedit.h>
#include <qspinbox.h>
#include <qmultilineedit.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qglobal.h>
#include <qobject.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qtoolbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>


#include <qpe/config.h>

#include "pocketcellar.h"
#include "cellarentry.h"
#include "wineentry.h"
#include "winechooser.h"
#include "cellareditor.h"
#include "winedb.h"
#include "cellardb.h"
#include "assistant.h"


void wineFromEditor(Wine *w, WineEditor *we) {
	w->setName(we->_name->text());
	w->setProducer(we->_producer->text());
	w->setRegion(we->_region->text());
	w->setYear(we->_year->value());
	w->setScore(we->_score->value());
	w->setReady(we->_ready->value());
	w->setNotes(we->_notes->text());
	w->setType(we->_type->currentItem());
	w->clearGrapes();
	for (uint i=0; i < we->_grapes->count(); i++)
		if (we->_grapes->isSelected(i))
			w->addGrape(we->_grapes->text(i));
}


void editorFromWine(WineEditor *we, Wine *w, bool ro = false) {
	we->_grapes->insertStringList(WineDB::grapeList);
	we->_name->setReadOnly(ro);
	we->_name->setText(w->name());
	we->_producer->setReadOnly(ro);
	we->_producer->setText(w->producer());
	we->_region->setReadOnly(ro);
	we->_region->setText(w->region());
	we->_notes->setReadOnly(ro);
	we->_notes->setText(w->notes());
	we->_type->setCurrentItem(w->type());
	we->_type->setEnabled(!ro);
	if (ro)
		we->_year->setRange(w->year(), w->year());
	we->_year->setValue(w->year());
	if (ro)
		we->_score->setRange(w->score(), w->score());
	we->_score->setValue(w->score());
	if (ro)
		we->_ready->setRange(w->ready(), w->ready());
	we->_ready->setValue(w->ready());
	if (ro)
		we->setCaption(QObject::tr("Wine Details"));
	if (ro)
		we->_grapes->viewport()->setEnabled(false);

	we->_grapes->clearSelection();
	for (uint i=0; i < w->grapes().count(); i++) {
		QListBoxItem *l = we->_grapes->findItem(w->grapes()[i]);
		if (l)
			we->_grapes->setSelected(l, true);
	}
}


PocketCellar::PocketCellar(QWidget *parent, const char *name)
	: PocketCellarBase(parent, name)
{
	Config cfg("pocketcellar");
	cfg.setGroup("General");
	int storeloc = cfg.readNumEntry("Data Location", 0);
	if (storeloc == 0)
		_store_group->setButton(_store_group->id(_store_builtin));
	else if (storeloc == 1)
		_store_group->setButton(_store_group->id(_store_cf));
	else if (storeloc == 2)
		_store_group->setButton(_store_group->id(_store_sd));
	WineDB::load(_databaseList, storeloc);
	WineDB::loadGrapeList();
	CellarDB::load(_cellarList, _databaseList, storeloc);
	connect(_cellarList, SIGNAL(selectionChanged(QListViewItem*)),
			this, SLOT(updateCellarButtons(QListViewItem*)));
	connect(_databaseList, SIGNAL(selectionChanged(QListViewItem*)),
			this, SLOT(updateDatabaseButtons(QListViewItem*)));
	setupAssistant();
	updateStats();
}

PocketCellar::~PocketCellar() {
	Config cfg("pocketcellar");
	cfg.setGroup("General");
	int storeloc = 0;
	if (_store_cf->isChecked())
		storeloc=1;
	else if (_store_sd->isChecked())
		storeloc=2;
//qDebug("storeloc=%d\n", storeloc);
	cfg.writeEntry("Data Location", storeloc);
	cfg.write();
	bool rc;
	rc = WineDB::save(_databaseList, storeloc);
//qDebug("Wine DB save - %s\n", rc ? "ok" : "failed");
	rc = CellarDB::save(_cellarList, storeloc);
//qDebug("Cellar DB save - %s\n", rc ? "ok" : "failed");
}



void PocketCellar::slotNewWine() {
	Wine *thisWine = 0;
	int rc = 1;
	
	if (_databaseList->childCount() > 0)
		rc = QMessageBox::information(this, tr("New Cellar Entry"),
					tr("Would you like to import\n"
					   "an existing wine or create\n"
					   "a new database entry?"),
					tr("&Import"), tr("&New"));

	if (rc == 0) {
		WineChooser *wc = new WineChooser(NULL, "Wine Chooser", true);
		for (QListViewItem *i = _databaseList->firstChild();
								  i;
						i = i->nextSibling()) {
			new WineChooserEntry(wc->_wines, static_cast<Wine*>(i));
		}
		wc->_wines->setSelected(wc->_wines->firstChild(), true);
		int rc = wc->exec();
		if (rc == QDialog::Accepted) {
			thisWine = static_cast<WineChooserEntry*>(wc->_wines->selectedItem())->_wine;
		}
		delete wc;
	} else if (rc == 1) {
		WineEditor *we = new WineEditor(NULL, "Wine Editor", true);
		we->_databaseList = _databaseList;
		we->_chooseRegion->setEnabled(_databaseList->childCount() > 0);
		we->_chooseProducer->setEnabled(_databaseList->childCount() > 0);
		we->showMaximized();
		we->_grapes->insertStringList(WineDB::grapeList);
		int rc = we->exec();
		if (rc == QDialog::Accepted) {
			Wine *w = new Wine(_databaseList);
			wineFromEditor(w, we);
			thisWine = w;
		}
		delete we;
	} else {  // rc == -1     -> abort
		return;
	}

	if (!thisWine)
		return;

	CellarEditor *ced = new CellarEditor(NULL, "Cellar Editor", true);
	ced->showMaximized();
	ced->_wine->setReadOnly(true);
	ced->_wine->setText(thisWine->name());
	ced->_producer->setReadOnly(true);
	ced->_producer->setText(thisWine->producer());
	ced->_year->setReadOnly(true);
	ced->_year->setText(QString::number(thisWine->year()));
	ced->_score->setReadOnly(true);
	ced->_score->setText(QString::number(thisWine->score()));
	rc = ced->exec();

	if (rc == QDialog::Accepted) {
		bool ok = false;
		CellarEntry *ce = new CellarEntry(_cellarList, thisWine);
		if (ced->_price->text().isEmpty())
			ce->setPrice(0.0);
		else ce->setPrice(ced->_price->text().toFloat(&ok));
		ce->setQuantity(ced->_quantity->value());
		ce->setNotes(ced->_notes->text());
		if (!ok && !ced->_price->text().isEmpty()) {
			QMessageBox::warning(NULL, tr("Price Error"), tr("The price you provided\ndoes not appear to be valid."), 1, 0, 0);
		}
		updateStats();
	}
	delete ced;
}


void PocketCellar::slotEditWine() {
	CellarEntry *ce = static_cast<CellarEntry*>(_cellarList->selectedItem());
	if (!ce)
		return;

	cellarEditor(ce, false);
	updateStats();
}


void PocketCellar::slotViewWine() {
	CellarEntry *ce = static_cast<CellarEntry*>(_cellarList->selectedItem());
	if (!ce)
		return;

	cellarEditor(ce, true);
}


void PocketCellar::cellarEditor(CellarEntry *ce, bool ro) {
	CellarEditor *ced = new CellarEditor(NULL, "Cellar Editor", true);
	if (ro) {
		ced->setCaption(ced->caption() + tr(" (read only)"));
	}
	ced->showMaximized();
	ced->_wine->setReadOnly(true);
	ced->_wine->setText(ce->wine()->name());
	ced->_producer->setReadOnly(true);
	ced->_producer->setText(ce->wine()->producer());
	ced->_year->setReadOnly(true);
	ced->_year->setText(QString::number(ce->wine()->year()));
	ced->_score->setReadOnly(true);
	ced->_score->setText(QString::number(ce->wine()->score()));
	ced->_price->setReadOnly(ro);
	ced->_price->setText(QString::number(ce->price()));
	ced->_notes->setReadOnly(ro);
	ced->_notes->setText(ce->notes());
	if (ro)
		ced->_quantity->setRange(ce->quantity(), ce->quantity());
	ced->_quantity->setValue(ce->quantity());
	int rc = ced->exec();
	if (!ro && rc == QDialog::Accepted) {
		bool ok = false;
		if (ced->_price->text().isEmpty())
			ce->setPrice(0.0);
		else ce->setPrice(ced->_price->text().toFloat(&ok));
		ce->setQuantity(ced->_quantity->value());
		ce->setNotes(ced->_notes->text());
		if (!ok && !ced->_price->text().isEmpty()) {
			QMessageBox::warning(NULL, tr("Price Error"), tr("The price you provided\ndoes not appear to be valid."), 1, 0, 0);
		}
		updateStats();
	}
	delete ced;
}


void PocketCellar::slotRemoveWine() {
	CellarEntry *e = static_cast<CellarEntry*>(_cellarList->selectedItem());
	if (!e)
		return;

	delete e;
	updateCellarButtons(_cellarList->selectedItem());
	updateStats();
}


void PocketCellar::slotNewWineDB() {
	WineEditor *we = new WineEditor(NULL, "Wine Editor", true);
	we->_databaseList = _databaseList;
	we->_chooseRegion->setEnabled(_databaseList->childCount() > 0);
	we->_chooseProducer->setEnabled(_databaseList->childCount() > 0);
	we->showMaximized();
	we->_grapes->insertStringList(WineDB::grapeList);
	int rc = we->exec();
	if (rc == QDialog::Accepted) {
		Wine *w = new Wine(_databaseList);
		wineFromEditor(w, we);
		updateStats();
	}
	delete we;
}


void PocketCellar::slotEditWineDB() {
	Wine *w = static_cast<Wine*>(_databaseList->selectedItem());
	if (!w)
		return;

	wineEditor(w, false);
	updateStats();
}


void PocketCellar::wineEditor(Wine *w, bool ro) {
	WineEditor *we = new WineEditor(NULL, "Wine Editor", true);
	we->_databaseList = _databaseList;
	if (ro) {
		we->setCaption(we->caption() + tr(" (read only)"));
	}
	we->_chooseRegion->setEnabled(!ro && _databaseList->childCount() > 0);
	we->_chooseProducer->setEnabled(!ro && _databaseList->childCount() > 0);
	we->showMaximized();
	editorFromWine(we, w, ro);
	int rc = we->exec();
	if (!ro && rc == QDialog::Accepted) {
		wineFromEditor(w, we);
		updateStats();
	}
	delete we;
}


void PocketCellar::slotRemoveWineDB() {
	Wine *w = static_cast<Wine*>(_databaseList->selectedItem());
	if (!w)
		return;

	CellarEntry *e = 0;
	for (e = static_cast<CellarEntry*>(_cellarList->firstChild());
								    e;
		       e = static_cast<CellarEntry*>(e->nextSibling())) {
		if (e->wine() == w)
			delete e;
	}
	delete w;
	updateDatabaseButtons(_databaseList->selectedItem());
	updateCellarButtons(_cellarList->selectedItem());
	updateStats();
}


void PocketCellar::slotViewWineDB() {
	Wine *w = static_cast<Wine*>(_databaseList->selectedItem());
	if (!w)
		return;

	wineEditor(w, true);
}


void PocketCellar::updateStats() {
	CellarEntry *e = 0;

	stats.clear();

	stats._wines = _databaseList->childCount();

	for (e = static_cast<CellarEntry*>(_cellarList->firstChild()); 
								    e; 
		      e = static_cast<CellarEntry*>(e->nextSibling())) {
		stats._bottles += e->quantity();
		if (stats._minPrice > e->price())
			stats._minPrice = e->price();
		if (stats._maxPrice < e->price())
			stats._maxPrice = e->price();
		stats._totalPrice += e->price()*e->quantity();

		Wine *w = e->wine();

		if (stats._minScore > w->score())
			stats._minScore = w->score();
		if (stats._maxScore < w->score())
			stats._maxScore = w->score();
		stats._totalScore += w->score()*e->quantity();
		if (stats._minAge > w->year())
			stats._minAge = w->year();
		if (stats._maxAge < w->year())
			stats._maxAge = w->year();
		stats._totalAge += w->year()*e->quantity();
	}

	_wines->setText(QString::number(stats._wines));
	_bottles->setText(QString::number(stats._bottles));
	QString tmp;
	tmp.sprintf(tr("$%.2f"), stats._totalPrice);
	_value->setText(tmp);
	tmp.sprintf(tr("$%.2f ... $%.2f, avg $%.2f"), 
			stats._minPrice <= stats._maxPrice ?
			stats._minPrice : 0.0,
			stats._maxPrice, stats._bottles > 0 ? 
			stats._totalPrice/stats._bottles :
			0);
	_prices->setText(tmp);
	tmp.sprintf(tr("%d ... %d, avg %d"), 
			stats._minScore <= stats._maxScore ? stats._minScore :0,
			stats._maxScore,
			stats._bottles > 0 ? stats._totalScore/stats._bottles :
			0);
	_scores->setText(tmp);
	tmp.sprintf(tr("%d ... %d, avg %d"),
			stats._minAge <= stats._maxAge ? stats._minAge : 0, 
			stats._maxAge,
			stats._bottles > 0 ? stats._totalAge/stats._bottles : 
			0);
	_ages->setText(tmp);
}


void PocketCellar::setupAssistant() {
	Config cfg("foodassist");
	cfg.setGroup("General");
	QStringList groups = cfg.readListEntry("Foods", ',');
	groups.sort();
	_assistantCombo->insertItem("");

	for (QStringList::Iterator i = groups.begin(); i != groups.end(); ++i) {
		cfg.setGroup(*i);
		grapeMap[*i] = cfg.readListEntry("Wine", ',');
		wineMap[*i] = cfg.readListEntry("Type", ',');
		_assistantCombo->insertItem(*i);
	}

	connect(_assistantCombo, SIGNAL(activated(const QString&)), this, SLOT(slotRecommend(const QString&)));
}


void PocketCellar::slotRecommend(const QString& wine) {
	_assistantGrapes->clear();
	_assistantWines->clear();
	if (!wine.isEmpty()) {
		_assistantGrapes->insertStringList(grapeMap[wine]);
		CellarEntry *e = 0;
		for (e = static_cast<CellarEntry*>(_cellarList->firstChild()); 
									    e; 
			      e = static_cast<CellarEntry*>(e->nextSibling())) {
			QStringList w = grapeMap[wine];
			QStringList t = wineMap[wine];
			QStringList g = e->wine()->grapes();
			for (QStringList::Iterator i = w.begin(); i != w.end();
									++i) {
				if (g.contains(*i)) {
					if (true /*t.contains(e->wine()->type())*/) {
						new AssistantEntry(_assistantWines, e);
						break;
					}
				}
			}
		}
	}
}


void PocketCellar::updateCellarButtons(QListViewItem* i) {
	_cellarEdit->setEnabled(i != 0);
	_cellarView->setEnabled(i != 0);
	_cellarRemove->setEnabled(i != 0);
}


void PocketCellar::updateDatabaseButtons(QListViewItem* i) {
	_databaseEdit->setEnabled(i != 0);
	_databaseView->setEnabled(i != 0);
	_databaseRemove->setEnabled(i != 0);
}


