/*
    PocketCellar

    Copyright (C) 2002 George Staikos <staikos@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "winedb.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>


Wine::Wine(QListView *parent)
: QListViewItem(parent) {
}

Wine::~Wine() {
}


QString& Wine::name() {
	return _name;
}

QString& Wine::region() {
	return _region;
}

QString& Wine::producer() {
	return _producer;
}

QStringList& Wine::grapes() {
	return _grapes;
}

Q_UINT16& Wine::score() {
	return _score;
}

Q_UINT16& Wine::ready() {
	return _ready;
}

Q_UINT16& Wine::year() {
	return _year;
}

unsigned char Wine::type() {
	return _type;
}

QString& Wine::notes() {
	return _notes;
}

void Wine::setName(QString name) {
	_name = name;
	setText(0, name);
}

void Wine::setProducer(QString producer) {
	_producer = producer;
	setText(1, producer);
}

void Wine::setRegion(QString region) {
	_region = region;
}

void Wine::addGrape(QString grape) {
	_grapes << grape;
}

void Wine::setGrapes(const QStringList& grapes) {
	_grapes = grapes;
}

void Wine::removeGrape(QString grape) {
	_grapes.remove(grape);
}

void Wine::clearGrapes() {
	_grapes.clear();
}

void Wine::setScore(Q_UINT16 score) {
	_score = score;
	setText(3, QString::number(score));
}

void Wine::setReady(Q_UINT16 ready) {
	_ready = ready;
}

void Wine::setYear(Q_UINT16 year) {
	_year = year;
	setText(2, QString::number(year));
}

void Wine::setType(unsigned char type) {
	_type = type;
}

void Wine::setNotes(QString notes) {
	_notes = notes;
}


Wine& Wine::operator=(const Wine& x) {
	setName(x._name);
	setProducer(x._producer);
	setRegion(x._region);
	setGrapes(x._grapes);
	setScore(x._score);
	setReady(x._ready);
	setYear(x._year);
	setType(x._type);
	setNotes(x._notes);
	return *this;
}




#include <qdatastream.h>
#include <qtextstream.h>
#include <qfile.h>


QDataStream& operator>>(QDataStream& s, Wine& w) {
QString str;
	s >> str;
	w.setName(str);
	s >> str;
	w.setProducer(str);
	s >> str;
	w.setRegion(str);
unsigned char c;
	s >> c;
	w.setType(c);

Q_UINT16 num;
	s >> num;
	w.setScore(num);
	s >> num;
	w.setReady(num);
	s >> num;
	w.setYear(num);

	s >> str;
	w.setNotes(str);

uint i;
	s >> i;
	w.clearGrapes();
	for (uint j = 0; j < i; j++) {
		QString g;
		s >> g;
		w.addGrape(g);
	}

return s;
}


QDataStream& operator<<(QDataStream& s, Wine& w) {
	s << w.name();
	s << w.producer();
	s << w.region();
	s << w.type();
	s << w.score();
	s << w.ready();
	s << w.year();
	s << w.notes();
	s << w.grapes().count();
	for (uint i = 0; i < w.grapes().count(); i++)
		s << w.grapes()[i];
return s;
}


QString Wine::uuid() const {
	QString rc;
	rc = _name + ":" + _producer + ":" + QString::number(_year);
	return rc;
}


int Wine::operator==(const QString& x) {
	return ((x == uuid()) ? 1 : 0);
}



bool WineDB::load(QListView *parent, int loc) {
	QString path("/home");
	
	if (loc == 1)
		path = "/usr/mnt.rom/cf";
	else if (loc == 2)
		path = "/usr/mnt.rom/card";

	mkdir(QString(path + "/QtPalmtop/data/PocketCellar").ascii(), 755);
	QFile f(path + "/QtPalmtop/data/PocketCellar/PCWineDB");

	if (!f.open(IO_ReadOnly))
		return false;

	QDataStream ds(&f);
	ds.setPrintableData(false);
	Q_UINT16 v;

	ds >> v;

	if (v != 1)
		return false;
	
	parent->clear();

	while (!ds.atEnd()) {
		Wine *w = new Wine(parent);
		ds >> *w;
	}

	f.close();

	if (loc != 0)
		unlink("/home/QtPalmtop/data/PocketCellar/PCWineDB");
	if (loc != 1)
		unlink("/usr/mnt.rom/cf/QtPalmtop/data/PocketCellar/PCWineDB");
	if (loc != 2)
		unlink("/usr/mnt.rom/card/QtPalmtop/data/PocketCellar/PCWineDB");
return true;
}


bool WineDB::save(QListView *parent, int loc) {
	QString path("/home");
	
	if (loc == 1)
		path = "/usr/mnt.rom/cf";
	else if (loc == 2)
		path = "/usr/mnt.rom/card";

	mkdir(QString(path + "/QtPalmtop/data/PocketCellar").ascii(), 755);
	QFile f(path + "/QtPalmtop/data/PocketCellar/PCWineDB");

	if (!f.open(IO_WriteOnly))
		return false;

	QDataStream ds(&f);
	ds.setPrintableData(false);
	
	ds << Q_UINT16(1);

	for (QListViewItem *i = parent->firstChild();
						   i;
				i = i->nextSibling()) {
		ds << *(static_cast<Wine*>(i));
	}

	f.close();

	if (loc != 0)
		unlink("/home/QtPalmtop/data/PocketCellar/PCWineDB");
	if (loc != 1)
		unlink("/usr/mnt.rom/cf/QtPalmtop/data/PocketCellar/PCWineDB");
	if (loc != 2)
		unlink("/usr/mnt.rom/card/QtPalmtop/data/PocketCellar/PCWineDB");
	return true;
}


QStringList WineDB::grapeList;

bool WineDB::loadGrapeList() {
	QFile f("/home/QtPalmtop/data/PocketCellar/grapelist.txt");

	if (!f.open(IO_ReadOnly))
		return false;

	QTextStream ts(&f);
	WineDB::grapeList.clear();

	while (!ts.atEnd()) {
		WineDB::grapeList << ts.readLine();
	}

return true;
}



WineChooserEntry::WineChooserEntry(QListView *parent, Wine *wine) 
: QListViewItem(parent), _wine(wine)
{
	setText(0, _wine->name());
	setText(1, _wine->producer());
	setText(2, QString::number(_wine->year()));
}


WineChooserEntry::~WineChooserEntry() {
}


