/*
    PocketCellar

    Copyright (C) 2002 George Staikos <staikos@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _WINEDB_H
#define _WINEDB_H

#include <qstring.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qmap.h>


class Wine : public QListViewItem {
	public:
		Wine(QListView *parent);
		virtual ~Wine();

		QString& name();
		QString& producer();
		QString& region();
		QStringList& grapes();
		Q_UINT16& score();
		Q_UINT16& ready();
		Q_UINT16& year();
		unsigned char type();
		QString& notes();

		
		void setName(QString name);
		void setProducer(QString producer);
		void setRegion(QString region);
		void addGrape(QString grape);
		void setGrapes(const QStringList& grapes);
		void removeGrape(QString grape);
		void clearGrapes();
		void setScore(Q_UINT16 score);
		void setReady(Q_UINT16 ready);
		void setYear(Q_UINT16 year);
		void setType(unsigned char type);
		void setNotes(QString notes);

		Wine& operator=(const Wine& x);
		int operator==(const QString& x);

		QString uuid() const;

	private:
		QString _name, _producer, _region, _notes;
		unsigned char _type;
		QStringList _grapes;
		Q_UINT16 _score, _ready, _year;
};



class WineChooserEntry : public QListViewItem {
	public:
		WineChooserEntry(QListView *parent, Wine *wine);
		virtual ~WineChooserEntry();

		Wine *_wine;
};



class WineDB : public QMap<long,Wine> {
	public:
//		static QMap<long,Wine*> wines;
		static long maxWine;
		static bool load(QListView *parent, int loc);
		static bool save(QListView *parent, int loc);
		static QStringList grapeList;
		static bool loadGrapeList();
};

#endif

